// STexture.cpp: Implementierung der Klasse STexture.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://www.steinsoft.net
//////////////////////////////////////////////////////////////////////

#include "STexture.h"

#define GL_CLAMP_TO_EDGE  0x812F

//Swap function

void swapBGRtoRGB(char* buffer,int size)
{
	for (int i = 0;i < size; i += 3)
	{
		buffer[i] ^= buffer[i+2] ^=
		buffer[i] ^= buffer[i+2];
	}
}

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

STexture::STexture()
{
	texture = 0;
	hBitmap = NULL;
}

STexture::~STexture()
{
	destroy();
}

STexture::STexture(const char* filename,int minFilter,int magFilter,
				   bool clampToEdge)
{
	createTexture(filename,minFilter,magFilter,clampToEdge);	
}


///////////////////////////////
// Funktionen
///////////////////////////////

bool STexture::createTexture(const char *filename, int minFilter, int magFilter,
				   bool clampToEdge)
{
	BITMAP textureBitmap;
	bool noError;

	destroy();

	noError = loadBitmap(filename,textureBitmap);

	if (noError)
	{
		//Swappen
		swapBGRtoRGB((char*)textureBitmap.bmBits,textureBitmap.bmHeight*
										  textureBitmap.bmWidth*
										  3);

		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		if (clampToEdge)
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
		}

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,textureBitmap.bmWidth,
			textureBitmap.bmHeight,0,GL_RGB,
			GL_UNSIGNED_BYTE,textureBitmap.bmBits);
		
		DeleteObject(hBitmap);
		
		return true;
	}

	
	
	return false;
}

bool STexture::createMipmap(const char *filename, int minFilter, int magFilter,
				   bool clampToEdge)
{
	BITMAP textureBitmap;
	bool noError;

	destroy();

	noError = loadBitmap(filename,textureBitmap);

	if (noError)
	{
		//Swappen
		swapBGRtoRGB((char*)textureBitmap.bmBits,textureBitmap.bmHeight*
										  textureBitmap.bmWidth*
										  3);
		
		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);
		
		if (clampToEdge)
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
		}
	
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB8,textureBitmap.bmWidth,
			textureBitmap.bmHeight,GL_RGB,
			GL_UNSIGNED_BYTE,textureBitmap.bmBits);

		DeleteObject(hBitmap);

		return true;
	}
	
	return false;
}

GLuint STexture::getGLTexture()
{
	return texture;
}

bool STexture::loadBitmap(const char *filename,BITMAP& bitmap)
{
	hBitmap = (HBITMAP)LoadImage(NULL,filename,IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION |
		LR_LOADFROMFILE);

	if (hBitmap)
	{
		GetObject(hBitmap,sizeof(bitmap),&bitmap);

		return true;
	}

	return false;
}

void STexture::select()
{
	glBindTexture(GL_TEXTURE_2D,texture);
}

void STexture::destroy()
{
	if (texture)
	{
		glDeleteTextures(1,&texture);
		texture = 0;
	}
}
